import asyncio
import logging

from loguru import logger

from embykeeper.log import formatter
from embykeeper.config import config

logger = logger.bind(scheme="telegram", nonotify=True)

stream_log = None
stream_msg = None
handler_log_id = None
handler_msg_id = None
change_handle_telegram = None
change_handle_notifier = None


async def _stop_notifier():
    global stream_log, stream_msg, handler_log_id, handler_msg_id

    if handler_log_id is not None:
        logger.remove(handler_log_id)
        handler_log_id = None
    if handler_msg_id is not None:
        logger.remove(handler_msg_id)
        handler_msg_id = None

    if stream_log:
        stream_log.close()
        await stream_log.join()
        stream_log = None
    if stream_msg:
        stream_msg.close()
        await stream_msg.join()
        stream_msg = None


def _handle_config_change(*args):
    async def _async():
        global stream_log, stream_msg

        await _stop_notifier()
        if config.notifier and config.notifier.enabled:
            streams = await start_notifier()
            if streams:
                stream_log, stream_msg = streams

    logger.debug("正在刷新 Telegram 消息通知.")
    asyncio.create_task(_async())


async def start_notifier():
    """消息通知初始化函数."""
    global stream_log, stream_msg, handler_log_id, handler_msg_id, change_handle_telegram, change_handle_notifier

    def _filter_log(record):
        notify = record.get("extra", {}).get("log", None)
        nonotify = record.get("extra", {}).get("nonotify", None)
        if (not nonotify) and (notify or record["level"].no == logging.ERROR):
            return True
        else:
            return False

    def _filter_msg(record):
        notify = record.get("extra", {}).get("msg", None)
        nonotify = record.get("extra", {}).get("nonotify", None)
        if (not nonotify) and notify:
            return True
        else:
            return False

    def _formatter(record):
        return "{level}#" + formatter(record)

    accounts = config.telegram.account
    notifier = config.notifier
    if notifier:
        account = None
        if notifier.enabled:
            if isinstance(notifier.account, int):
                try:
                    account = accounts[notifier.account - 1]
                except IndexError:
                    notifier = None
            elif isinstance(notifier, str):
                for a in accounts:
                    if a.phone == notifier:
                        account = a
                        break

    if account:
        from .session import ClientsSession
        from .log import TelegramStream

        async with ClientsSession([account]) as clients:
            async for a, tg in clients:
                logger.info(f'计划任务的关键消息将通过 Embykeeper Bot 发送至 "{account.phone}" 账号.')
                break
            else:
                logger.error(f'无法连接到 "{account.phone}" 账号, 无法发送日志推送.')
                return None

        stream_log = TelegramStream(
            account=account,
            instant=config.notifier.immediately,
        )
        handler_log_id = logger.add(
            stream_log,
            format=_formatter,
            filter=_filter_log,
        )
        stream_msg = TelegramStream(
            account=account,
            instant=True,
        )
        handler_msg_id = logger.add(
            stream_msg,
            format=_formatter,
            filter=_filter_msg,
        )
        if not change_handle_telegram:
            change_handle_telegram = config.on_change("telegram.account", _handle_config_change)
        if not change_handle_notifier:
            change_handle_notifier = config.on_change("notifier", _handle_config_change)
        return stream_log, stream_msg
    else:
        if not change_handle_notifier:
            change_handle_notifier = config.on_change("notifier", _handle_config_change)
        return None



import asyncio
import random
import string
from urllib.parse import parse_qs, urlparse

from faker import Faker
import httpx
from pyrogram.types import Message
from pyrogram import filters
from pyrogram.raw.functions.messages import RequestWebView
from pyrogram.errors import MessageIdInvalid

from embykeeper.config import config
from embykeeper.utils import get_proxy_str

from ..link import Link
from . import Monitor

misty_monitor_pool = {}


class WokeMonitor(Monitor):
    name = "蜗壳"
    chat_name = "Walking_Chat"
    chat_keyword = r"可注册人数: (?!0$)"
    bot_username = "Readsnail_bot"
    notify_create_name = True
    additional_auth = ["prime"]
    allow_edit = False

    async def solve_captcha(self, url: str):
        token = await Link(self.client).captcha("woke")
        if not token:
            return False
        else:
            scheme = urlparse(url)
            params = parse_qs(scheme.query)
            url_submit = scheme._replace(path="/api/verify", query="", fragment="").geturl()
            uuid = params.get("id", [None])[0]
            origin = scheme._replace(path="/", query="", fragment="").geturl()
            useragent = Faker().safari()
            headers = {
                "Content-Type": "application/x-www-form-urlencoded",
                "Referer": url,
                "Origin": origin,
                "User-Agent": useragent,
            }
            data = {
                "uuid": uuid,
                "cf-turnstile-response": token,
            }
            try:
                async with httpx.AsyncClient(http2=True, proxy=get_proxy_str(config.proxy)) as client:
                    resp = await client.post(url_submit, headers=headers, data=data)
                    result = resp.text
                    if "完成" in result:
                        return True
            except:
                return False

    async def on_trigger(self, message: Message, key, reply):
        for i in range(3):
            await self.client.send_message(self.bot_username, "/cancel")
            await asyncio.sleep(random.uniform(1, 2))
            if i:
                self.log.info(f"正在重试注册 ({i}/3).")
            try:
                msg_start = await self.client.wait_reply(self.bot_username, f"/start")
            except asyncio.TimeoutError:
                self.log.warning("发送消息无响应, 无法注册.")
                return
            text = msg_start.text or msg_start.caption
            if not "您还未注册" in text:
                self.log.info("账户已注册, 无需抢注.")
                return
            async with self.client.catch_reply(self.bot_username) as f:
                try:
                    msg = await msg_start.click("创建账户", timeout=1)
                except (TimeoutError, MessageIdInvalid):
                    pass
                except ValueError:
                    self.log.warning("未能找到注册按钮, 无法注册.")
                    return
                try:
                    msg: Message = await asyncio.wait_for(f, timeout=10)
                except asyncio.TimeoutError:
                    self.log.warning("点击注册按钮无响应, 无法注册.")
                    return
            text = msg.text or msg.caption
            if msg.text and "先加入" in msg.text:
                self.log.warning("账户错误, 无法注册.")
                continue
            elif "完成验证" in text:
                self.log.info("需要验证, 正在解析.")
                url = None
                if msg.reply_markup:
                    buttons = [button for line in msg.reply_markup.inline_keyboard for button in line]
                    for b in buttons:
                        if "验证" in b.text and b.web_app:
                            url = b.web_app.url
                            break
                if not url:
                    self.log.warning("需要验证身份但没有找到 URL, 无法注册.")
                    return
                bot_peer = await self.client.resolve_peer(self.bot_username)
                url_auth = (
                    await self.client.invoke(
                        RequestWebView(peer=bot_peer, bot=bot_peer, platform="ios", url=url)
                    )
                ).url
                if not await self.solve_captcha(url_auth):
                    self.log.warning("验证码解析失败, 无法注册.")
                    return
                else:
                    await asyncio.sleep(random.uniform(3, 5))
                    self.log.info("已成功验证, 继续进行注册流程.")
                async with self.client.catch_reply(
                    self.bot_username, filter=~filters.regex("请耐心等待")
                ) as f:
                    try:
                        msg = await msg_start.click("创建账户", timeout=1)
                    except (TimeoutError, MessageIdInvalid):
                        pass
                    except ValueError:
                        self.log.warning("未能找到注册按钮, 无法注册.")
                        return
                    try:
                        msg: Message = await asyncio.wait_for(f, timeout=10)
                    except asyncio.TimeoutError:
                        self.log.warning("点击注册按钮无响应, 无法注册.")
                        return
                    if "可注册名额不足" in msg:
                        self.log.warning("名额不足, 无法注册.")
                        return
                    else:
                        self.log.bind(msg=True).info(f"已在 Bot @{self.bot_username} 成功创建用户, 请查看.")
                        return



from . import BotCheckin

__ignore__ = True


class DataSGKCheckin(BotCheckin):
    name = "数据社工库"
    bot_username = "DataSGK_bot"
    bot_checkin_cmd = "/signin"
    bot_checked_keywords = "请勿重复签到"



import asyncio
from contextlib import asynccontextmanager
from datetime import date, datetime, time, timedelta
from pathlib import Path
import random
import sys
import site
import traceback
from typing import Any, Coroutine, Iterable, Optional, Union

from loguru import logger

from . import __url__, __name__, __version__
from .schema import ProxyConfig


def get_path_frame(e, path):
    """获取指定路径下的最后一个错误栈帧.

    Args:
        e: 异常对象
        path: 要搜索的路径

    Returns:
        FrameSummary: 找到的栈帧, 如果未找到则返回 None
    """
    try:
        tb = traceback.extract_tb(e.__traceback__)
        for frame in reversed(tb):
            if Path(path) in Path(frame.filename).parents and frame.name not in ("invoke", "__getattr__"):
                return frame
        else:
            return None
    except AttributeError:
        return None


def get_last_frame(e):
    """获取异常的最后一个栈帧.

    Args:
        e: 异常对象

    Returns:
        FrameSummary: 最后一个栈帧, 如果未找到则返回 None
    """
    try:
        tb = traceback.extract_tb(e.__traceback__)
        for frame in reversed(tb):
            return frame
    except AttributeError:
        return None


def get_cls_fullpath(c):
    """获取类的完整路径名称.

    Args:
        c: 类对象

    Returns:
        str: 类的完整路径名称, 包含模块名
    """
    module = c.__module__
    if module == "builtins":
        return c.__qualname__
    return module + "." + c.__qualname__


def format_exception(e, regular=True):
    """格式化异常信息为可读字符串.

    Args:
        e: 异常对象
        regular: 是否为常规异常 (如网络错误等), 影响不同日志等级下提示信息的格式

    Returns:
        str: 格式化后的异常信息
    """
    if not regular:
        prompt = f"\n请在 Github 或交流群反馈下方错误详情以帮助开发者修复该问题 (当前版本: {__version__}):\n"
    else:
        prompt = ""
    proj_path = Path(__file__).parent.absolute()
    proj_frame = get_path_frame(e, proj_path)
    if proj_frame:
        proj_frame_path = Path(proj_frame.filename).relative_to(proj_path)
        prompt += f"\n  P {proj_frame_path}:{proj_frame.lineno}, F {proj_frame.name}:"
        prompt += f"\n    {proj_frame.line.strip()}"
    last_frame = get_last_frame(e)
    if last_frame:
        last_frame_path = last_frame.filename
        for p in site.getsitepackages():
            if Path(p) in Path(last_frame.filename).parents:
                last_frame_path = "<SP>/" + str(Path(last_frame.filename).relative_to(p))
                break
        prompt += f"\n  S {last_frame_path}:{last_frame.lineno}, F {last_frame.name}:"
        prompt += f"\n    {last_frame.line.strip()}"
    prompt += f"\n    E {get_cls_fullpath(type(e))}: {e}\n"
    return prompt


def show_exception(e, regular=True):
    """显示异常信息.

    Args:
        e: 异常对象
        regular: 是否为常规异常 (如网络错误等), 影响不同日志等级下提示信息的格式
    """
    from . import var

    if (regular and var.debug <= 1) or (not regular and var.debug == 0):
        var.console.rule()
        print(format_exception(e, regular=regular), flush=True, file=sys.stderr)
        var.console.rule()
    else:
        logger.opt(exception=e).debug("错误详情:")


class AsyncTaskPool:
    """一个用于批量等待异步任务的管理器, 支持在等待时添加任务."""

    def __init__(self):
        self.waiter = asyncio.Condition()
        self.tasks = []

    def add(self, coro: Coroutine, name: str = None):
        async def wrapper():
            task = asyncio.ensure_future(coro)
            await asyncio.wait([task])
            async with self.waiter:
                self.waiter.notify()
                return await task

        t = asyncio.create_task(wrapper())
        t.set_name(name or coro.__name__)
        self.tasks.append(t)
        return t

    async def as_completed(self):
        for t in self.tasks:
            if t.done():
                yield t
                self.tasks.remove(t)
        while self.tasks:
            async with self.waiter:
                await self.waiter.wait()
                for t in self.tasks[:]:
                    if t.done():
                        yield t
                        self.tasks.remove(t)

    async def wait(self):
        results = []
        async for t in self.as_completed():
            results.append(t.result())
        return results


class AsyncCountPool(dict):
    """一个异步安全的 ID 分配器.

    Args:
        base: ID 起始数
    """

    def __init__(self, *args, base=1000, **kw):
        super().__init__(*args, **kw)
        self.lock = asyncio.Lock()
        self.next = base + 1

    async def append(self, value):
        """输入一个值, 该值将被存储并分配一个 ID."""
        async with self.lock:
            key = self.next
            self[key] = value
            self.next += 1
            return key


def to_iterable(var: Union[Iterable, Any]):
    """
    将任何变量变为可迭代变量.

    Note:
        None 将变为空数组.
        非可迭代变量将变为仅有该元素的长度为 1 的数组.
        可迭代变量将保持不变.
    """
    if var is None:
        return ()
    if isinstance(var, str) or not isinstance(var, Iterable):
        return (var,)
    else:
        return var


def remove_prefix(text: str, prefix: str):
    """从字符串头部去除前缀."""
    return text[text.startswith(prefix) and len(prefix) :]


def truncate_str(text: str, length: int):
    """将字符串截断到特定长度, 并增加"..."后缀."""
    return f"{text[:length + 3]}..." if len(text) > length else text


def time_in_range(start, end, x):
    """判定时间在特定范围内."""
    if start <= end:
        return start <= x <= end
    else:
        return start <= x or x <= end


def batch(iterable, n=1):
    """将数组分成 N 份."""
    l = len(iterable)
    for ndx in range(0, l, n):
        yield iterable[ndx : min(ndx + n, l)]


def flatten(l):
    """将二维数组变为一维数组."""
    return [item for sublist in l for item in sublist]


def async_partial(f, *args1, **kw1):
    """Partial 函数的异步形式."""

    async def func(*args2, **kw2):
        return await f(*args1, *args2, **kw1, **kw2)

    return func


async def idle():
    """异步无限等待函数."""
    await asyncio.Future()


def random_time(start_time: time = None, end_time: time = None):
    """在特定的开始和结束时间之间生成时间, 如果开始时间晚于结束时间, 视为过夜."""
    start_datetime = datetime.combine(date.today(), start_time or time(0, 0))
    end_datetime = datetime.combine(date.today(), end_time or time(23, 59, 59))
    if end_datetime < start_datetime:
        end_datetime += timedelta(days=1)
    time_diff_seconds = (end_datetime - start_datetime).total_seconds()
    random_seconds = random.randint(0, int(time_diff_seconds))
    random_time = (start_datetime + timedelta(seconds=random_seconds)).time()
    return random_time


def next_random_datetime(start_time: time = None, end_time: time = None, interval_days: int = 1):
    """在特定的开始和结束时间之间生成时间, 并设定最小间隔天数."""
    if interval_days == 0:
        min_datetime = datetime.now()
    else:
        min_date = (datetime.now() + timedelta(days=interval_days)).date()
        min_datetime = datetime.combine(min_date, time(0, 0))
    target_time = random_time(start_time, end_time)
    offset_date = 0
    while True:
        t = datetime.combine(datetime.now() + timedelta(days=offset_date), target_time)
        if t >= min_datetime:
            break
        else:
            offset_date += 1
    return t


def format_timedelta_human(delta: timedelta):
    """将时间差转换为人类可读形式."""
    d = delta.days
    h, s = divmod(delta.seconds, 3600)
    m, s = divmod(s, 60)
    labels = ["天", "小时", "分钟", "秒"]
    dhms = ["%s %s" % (i, lbl) for i, lbl in zip([d, h, m, s], labels)]
    for start in range(len(dhms)):
        if not dhms[start].startswith("0"):
            break
    for end in range(len(dhms) - 1, -1, -1):
        if not dhms[end].startswith("0"):
            break
    parts = dhms[start : end + 1]
    if not parts:
        return "0 秒"
    else:
        return ", ".join(parts)


def format_byte_human(B: float):
    """将字节数转换为人类可读形式."""
    B = float(B)
    KB = float(1024)
    MB = float(KB**2)  # 1,048,576
    GB = float(KB**3)  # 1,073,741,824
    TB = float(KB**4)  # 1,099,511,627,776

    if B < KB:
        return "{0} {1}".format(B, "Bytes" if 0 == B > 1 else "Byte")
    elif KB <= B < MB:
        return "{0:.2f} KB".format(B / KB)
    elif MB <= B < GB:
        return "{0:.2f} MB".format(B / MB)
    elif GB <= B < TB:
        return "{0:.2f} GB".format(B / GB)
    elif TB <= B:
        return "{0:.2f} TB".format(B / TB)


@asynccontextmanager
async def nonblocking(lock: asyncio.Lock):
    """如果锁需要等待释放, 就跳过该部分."""
    try:
        await asyncio.wait_for(lock.acquire(), 0)
    except asyncio.TimeoutError:
        acquired = False
    else:
        acquired = True
    try:
        yield
    finally:
        if acquired:
            lock.release()


@asynccontextmanager
async def optional(lock: Optional[asyncio.Lock]):
    """可选的异步锁, 锁为 None 就直接运行该部分."""
    if lock is None:
        yield
    else:
        async with lock:
            yield


def distribute_numbers(min_value, max_value, num_elements=1, min_distance=0, max_distance=None, base=[]):
    """随机将一定数量的元素分布在最大最小值之间, 同时限定两元素之间的最小距离和最大距离, 生成起始元素由 `base` 定义."""
    if max_value < min_value:
        raise ValueError("invalid value range.")

    if max_distance and max_distance < min_distance:
        raise ValueError("invalid distance range.")

    numbers = sorted(base)
    results = []

    for _ in range(num_elements):
        allowed_range = []
        for i in range(-1, len(numbers)):
            if i == -1:
                min_allowed_value = min_value
            else:
                min_allowed_value = max(numbers[i] + min_distance, min_value)
            if i == len(numbers) - 1:
                max_allowed_value = max_value
            else:
                max_allowed_value = min(numbers[i + 1] - min_distance, max_value)
            if min_allowed_value < max_allowed_value:
                allowed_range.append((min_allowed_value, max_allowed_value))
        if not allowed_range:
            break

        # Calculate estimated elements for each range
        estimated_num_elements = [
            max(1, min(int((r[1] - r[0]) // min_distance), num_elements)) for r in allowed_range
        ]

        # Select a range using the estimated numbers as weights
        r = random.choices(allowed_range, k=1, weights=estimated_num_elements)[0]
        d = r[1] - r[0]
        min_v = r[0] + min_distance if r[0] == min_value else r[0]
        max_v = r[1]
        if max_distance and d > max_distance:
            value = random.uniform(min_v, r[0] + max_distance - min_distance)
        else:
            value = random.uniform(min_v, max_v)
        numbers = sorted(numbers + [value])
        results.append(value)
    return sorted(results)


def get_proxy_str(proxy: Optional[ProxyConfig] = None, curl: bool = False):
    """将代理设置转为 URL 形式."""
    if proxy:
        if curl and proxy.scheme == "socks5":
            schema = "socks5h"
        else:
            schema = proxy.scheme
        proxy_str = f"{schema}://"
        if proxy.username:
            proxy_str += f"{proxy.username or ''}:{proxy.password or ''}@"
        proxy_str += f"{proxy.hostname}:{proxy.port}"
    else:
        proxy_str = None
    return proxy_str


def deep_update(base_dict, update_dict):
    """递归地更新字典"""
    for key, value in update_dict.items():
        if isinstance(value, dict) and key in base_dict and isinstance(base_dict[key], dict):
            deep_update(base_dict[key], value)
        else:
            base_dict[key] = value
    return base_dict


class ProxyBase:
    """
    A proxy class that make accesses just like direct access to __subject__ if not overwriten in the class.
    Attributes defined in class. attrs named in __noproxy__ will not be proxied to __subject__.
    """

    __slots__ = ()

    def __call__(self, *args, **kw):
        return self.__subject__(*args, **kw)

    def hasattr(self, attr):
        try:
            object.__getattribute__(self, attr)
            return True
        except AttributeError:
            return False

    def __getattribute__(self, attr, oga=object.__getattribute__):
        if attr.startswith("__") and attr not in oga(self, "_noproxy"):
            subject = oga(self, "__subject__")
            if attr == "__subject__":
                return subject
            return getattr(subject, attr)
        return oga(self, attr)

    def __getattr__(self, attr, oga=object.__getattribute__):
        if attr == "hasattr" or self.hasattr(attr):
            return oga(self, attr)
        else:
            return getattr(oga(self, "__subject__"), attr)

    @property
    def _noproxy(self, oga=object.__getattribute__):
        import inspect

        base = oga(self, "__class__")
        for cls in inspect.getmro(base):
            if hasattr(cls, "__noproxy__"):
                yield from cls.__noproxy__

    def __setattr__(self, attr, val, osa=object.__setattr__):
        if attr == "__subject__" or attr in self._noproxy:
            return osa(self, attr, val)
        return setattr(self.__subject__, attr, val)

    def __delattr__(self, attr, oda=object.__delattr__):
        if attr == "__subject__" or hasattr(type(self), attr) and not attr.startswith("__"):
            oda(self, attr)
        else:
            delattr(self.__subject__, attr)

    def __bool__(self):
        return bool(self.__subject__)

    def __getitem__(self, arg):
        return self.__subject__[arg]

    def __setitem__(self, arg, val):
        self.__subject__[arg] = val

    def __delitem__(self, arg):
        del self.__subject__[arg]

    def __getslice__(self, i, j):
        return self.__subject__[i:j]

    def __setslice__(self, i, j, val):
        self.__subject__[i:j] = val

    def __delslice__(self, i, j):
        del self.__subject__[i:j]

    def __contains__(self, ob):
        return ob in self.__subject__

    for name in "repr str hash len abs complex int long float iter".split():
        exec("def __%s__(self): return %s(self.__subject__)" % (name, name))

    for name in "cmp", "coerce", "divmod":
        exec("def __%s__(self, ob): return %s(self.__subject__, ob)" % (name, name))

    for name, op in [
        ("lt", "<"),
        ("gt", ">"),
        ("le", "<="),
        ("ge", ">="),
        ("eq", " == "),
        ("ne", "!="),
    ]:
        exec("def __%s__(self, ob): return self.__subject__ %s ob" % (name, op))

    for name, op in [("neg", "-"), ("pos", "+"), ("invert", "~")]:
        exec("def __%s__(self): return %s self.__subject__" % (name, op))

    for name, op in [
        ("or", "|"),
        ("and", "&"),
        ("xor", "^"),
        ("lshift", "<<"),
        ("rshift", ">>"),
        ("add", "+"),
        ("sub", "-"),
        ("mul", "*"),
        ("div", "/"),
        ("mod", "%"),
        ("truediv", "/"),
        ("floordiv", "//"),
    ]:
        exec(
            (
                "def __%(name)s__(self, ob):\n"
                "    return self.__subject__ %(op)s ob\n"
                "\n"
                "def __r%(name)s__(self, ob):\n"
                "    return ob %(op)s self.__subject__\n"
                "\n"
                "def __i%(name)s__(self, ob):\n"
                "    self.__subject__ %(op)s=ob\n"
                "    return self\n"
            )
            % locals()
        )

    del name, op

    def __index__(self):
        return self.__subject__.__index__()

    def __rdivmod__(self, ob):
        return divmod(ob, self.__subject__)

    def __pow__(self, *args):
        return pow(self.__subject__, *args)

    def __ipow__(self, ob):
        self.__subject__ **= ob
        return self

    def __rpow__(self, ob):
        return pow(ob, self.__subject__)


class Proxy(ProxyBase):
    def __init__(self, val):
        self.set(val)

    def set(self, val):
        self.__subject__ = val


class FuncProxy(ProxyBase):
    __noproxy__ = ("_func", "_args", "_kw")

    def __init__(self, func, *args, **kw):
        self._func = func
        self._args = args
        self._kw = kw

    @property
    def __subject__(self):
        return self._func(*self._args, **self._kw)


class CachedFuncProxy(FuncProxy):
    __noproxy__ = ("_cached_value",)

    def __init__(self, func, *args, **kw):
        super().__init__(func, *args, **kw)
        self._cached_value = None

    @property
    def __subject__(self):
        if self._cached_value is None:
            self._cached_value = self._func(*self._args, **self._kw)
        return self._cached_value



import asyncio

from ..link import Link
from ..lock import pornfans_alert, pornfans_messager_mids_lock, pornfans_messager_mids
from . import BotCheckin

__ignore__ = True


class PornfansGameGroupCheckin(BotCheckin):
    name = "PornFans 游戏群发言"
    chat_name = "embytestflight"
    additional_auth = ["pornemby_pack"]
    bot_use_captcha = False

    async def send_checkin(self, retry=False):
        if pornfans_alert.get(self.client.me.id, False):
            self.log.warning("签到失败: 由于风险急停不进行发言")
            return await self.fail(message="由于风险急停不进行发言")

        for _ in range(3):
            min_letters = self.config.get("min_letters", self.config.get("letters", 8))
            max_letters = self.config.get("max_letters", 15)
            prompt = self.config.get(
                "prompt",
                f"请输出{min_letters}个字以上, {max_letters}个字以下的中文回复. 你需要进行一个群组聊天中的发言,"
                "表示的意思是 '发言换取答题资格', 你可以口语化一点, 像真人会说的话, 或者有水群一下的意思."
                "必须严格遵守字数要求！禁止输出逗号句号，开头必须有标号'@@@' (不计入字数)",
            )
            answer, by = await Link(self.client).gpt(prompt)
            if not answer:
                continue

            # Extract first valid response that meets the criteria
            valid_messages = [
                l.lstrip("@@@").strip()
                for l in answer.splitlines()
                if l.startswith("@@@") and min_letters <= len(l.lstrip("@@@").strip()) <= max_letters
            ]

            if not valid_messages:
                continue

            message = valid_messages[0]
            self.log.info(f"即将向群组发送水群消息: {message}.")

            await asyncio.sleep(10)
            if retry:
                await asyncio.sleep(self.bot_retry_wait)

            message = await self.send(message)
            if not message:
                await self.fail(message="发送失败")
            else:
                async with pornfans_messager_mids_lock:
                    if self.client.me.id not in pornfans_messager_mids:
                        pornfans_messager_mids[self.me.id] = []
                pornfans_messager_mids[self.me.id].append(message.id)
                await self.finish(message="已发送发言")
            return
        else:
            return await self.fail(message="无法生成发言内容")

    async def message_handler(*args, **kw):
        return



from asyncio import Event
from rich.console import Console

debug = 0
console = Console(stderr=True)
tele_used = Event()
emby_used = Event()
subsonic_used = Event()
exit_handlers = []
use_mongodb_config = False



import asyncio

from cachetools import TTLCache

ocrs = TTLCache(maxsize=1024, ttl=3600)  # spec: (DdddOcr, bool)
ocrs_lock = asyncio.Lock()

misty_monitors = {}  # uid: MistyMonitor
misty_locks = {}  # uid: lock

pornfans_nohp = {}  # uid: date
pornfans_messager_enabled = {}  # uid: bool
pornfans_alert = {}  # uid: bool
pornfans_messager_mids = {}  # uid: list(mid)
pornfans_messager_mids_lock = asyncio.Lock()

super_ad_shown = {}  # uid: bool
super_ad_shown_lock = asyncio.Lock()

authed_services = {}  # uid: {service: bool}
authed_services_lock = asyncio.Lock()



from .pornfans_double import PornfansDoubleMonitor

__ignore__ = True


class TestPornfansDoubleMonitor(PornfansDoubleMonitor):
    name = "PornFans 怪兽自动翻倍 测试"
    chat_name = "api_group"
    chat_user = "embykeeper_test_bot"
    chat_allow_outgoing = True



import asyncio
from datetime import datetime
import random
import re
from typing import List, Optional, Union

from cachetools import TTLCache
from loguru import logger
from pyrogram import Client
from pyrogram.enums import ChatType
from pyrogram.types import Message
from pyrogram.errors import RPCError
from cachetools import TTLCache
from pydantic import BaseModel, ValidationError

from embykeeper.runinfo import RunStatus
from embykeeper.utils import to_iterable, truncate_str

from . import Monitor

__ignore__ = True


class TemplateBMonitorConfig(BaseModel):
    name: str = None
    chat_name: Optional[Union[str, int]] = None  # 监控的群聊名称
    chat_follow_user: int = 10  # 需要等待 N 个用户发送相同内容方可回复
    chat_keyword: Union[str, List[str]] = []  # 仅当消息含有列表中的关键词时触发, 支持 regex
    chat_except_keyword: Union[str, List[str]] = []  # 消息含有列表中的关键词时不触发, 支持 regex
    chat_probability: float = 1.0  # 发信概率 (0最低, 1最高)
    chat_delay: int = 0  # 发信延迟 (秒)
    chat_max_length: int = 120  # 发送最大长度 (字符)
    trigger_interval: float = 120  # 每次触发的最低时间间隔 (秒)
    allow_same_user: bool = False  # 是否允许同一个人的消息


class TemplateBMonitor(Monitor):
    init_first = True
    additional_auth = ["prime"]

    async def init(self):
        try:
            self.t_config = TemplateBMonitorConfig.model_validate(self.config)
        except ValidationError as e:
            self.log.warning(f"初始化失败: 监控器自定义模板 B 的配置错误:\n{e}")
            return False
        self.name = self.t_config.name or "自定义"
        self.chat_name = self.t_config.chat_name
        self.chat_keyword = self.t_config.chat_keyword
        self.chat_except_keyword = self.t_config.chat_except_keyword
        self.chat_probability = self.t_config.chat_probability
        self.chat_delay = self.t_config.chat_delay
        self.chat_max_length = self.t_config.chat_max_length
        self.chat_follow_user = self.t_config.chat_follow_user
        self.trigger_interval = self.t_config.trigger_interval
        self.allow_same_user = self.t_config.allow_same_user

        self.lock = asyncio.Lock()
        self.chat_history = TTLCache(maxsize=2048, ttl=300)
        self.last_send = None
        self.log = logger.bind(scheme="telemonitor", name=self.name, username=self.client.me.full_name)
        return True

    async def message_handler(self, client: Client, message: Message):
        if not message.text:
            return
        if message.chat.type not in (ChatType.GROUP, ChatType.SUPERGROUP):
            return
        if len(message.text) > 50:
            return
        if message.text.startswith("/"):
            return
        if not message.from_user:
            return
        if message.from_user.is_bot:
            return
        for k in to_iterable(self.chat_except_keyword):
            if re.search(k, message.text, re.IGNORECASE):
                return
        if self.chat_keyword:
            for k in to_iterable(self.chat_keyword):
                if re.search(k, message.text, re.IGNORECASE):
                    break
            else:
                return
        if self.chat_max_length and len(message.text) > self.chat_max_length:
            return
        ident = (message.chat.id, message.text)
        async with self.lock:
            if ident not in self.chat_history:
                self.chat_history[ident] = {message.from_user.id} if not self.allow_same_user else 1
                return

            if self.allow_same_user:
                self.chat_history[ident] += 1
                count = self.chat_history[ident]
            else:
                self.chat_history[ident].add(message.from_user.id)
                count = len(self.chat_history[ident])

            if count == self.chat_follow_user:
                try:
                    chat_id, text = ident
                    if random.random() >= self.chat_probability:
                        return
                    if self.chat_delay and self.chat_delay > 0:
                        await asyncio.sleep(self.chat_delay)
                    if self.trigger_interval and self.trigger_interval > 0:
                        if self.last_send:
                            elapsed_time = (datetime.now() - self.last_send).total_seconds()
                            if self.last_send and (elapsed_time < self.trigger_interval):
                                await asyncio.sleep(self.trigger_interval - elapsed_time)
                    await self.client.send_message(chat_id, text)
                except RPCError as e:
                    self.log.warning(f"发送从众信息到群组 {message.chat.title} 失败: {e}.")
                else:
                    self.last_send = datetime.now()
                    self.log.info(f"已发送从众信息到群组 {message.chat.title}: {text}.")


def use(**kw):
    return type("TemplatedClass", (TemplateBMonitor,), kw)



import asyncio
import random
from . import BotCheckin

from pyrogram.types import Message, InlineKeyboardMarkup
from pyrogram.raw.types.messages import BotCallbackAnswer

__ignore__ = True


class MiniSGKCheckin(BotCheckin):
    name = "迷你世界社工库"
    bot_username = "mnsjsgkbot"
    bot_checkin_cmd = "/sign"

    async def message_handler(self, client, message: Message):
        if (
            message.reply_markup
            and isinstance(message.reply_markup, InlineKeyboardMarkup)
            and message.reply_markup.inline_keyboard
        ):
            keys = [k.text for r in message.reply_markup.inline_keyboard for k in r]
            for k in keys:
                if "签到" in k:
                    await asyncio.sleep(random.uniform(0.5, 1.5))
                    answer: BotCallbackAnswer = await message.click(k)
                    await self.on_text(Message(id=0), answer.message)
                    return
            else:
                self.log.warning(f"签到失败: 账户错误.")
                return await self.fail()



from ._templ_a import TemplateACheckin


class NanguaCheckin(TemplateACheckin):
    name = "南瓜音乐"
    bot_username = "nanguaemby_bot"
    bot_success_pat = "签到成功"
    bot_account_fail_keywords = ["你有号吗你就签到"]



from datetime import datetime
from urllib.parse import parse_qs, urlparse
import hmac
import hashlib
import random
import string

from pyrogram.types import Message
from pyrogram.raw.functions.messages import RequestWebView
from faker import Faker
import httpx

from embykeeper.config import config
from embykeeper.utils import to_iterable, truncate_str, get_proxy_str
from embykeeper.runinfo import RunStatus

from ..link import Link
from ._templ_a import TemplateACheckin


class RujiCheckin(TemplateACheckin):
    name = "入机"
    bot_username = "ljembyfukh_bot"
    bot_use_captcha = False
    bot_checkin_cmd = "/start"
    additional_auth = ["captcha"]

    signing_secret = "yNPEhmtaRwxYxk0LABf-pCeU6LlmE3CikoPej-g6xpQ"

    def generate_nonce(self, length=21):
        """Generate a random nonce string of specified length."""
        return "".join(random.choices(string.ascii_lowercase + string.digits, k=length))

    def generate_signature(self, user_id: int, timestamp: int, nonce: str) -> str:
        """Generate HMAC signature using SHA256."""
        message = f"{user_id}:{timestamp}:{nonce}"
        hmac_obj = hmac.new(self.signing_secret.encode(), message.encode(), hashlib.sha256)
        return hmac_obj.hexdigest()

    async def message_handler(self, client, message: Message):
        text = message.caption or message.text
        if (
            text
            and any(keyword in text for keyword in to_iterable(self.templ_panel_keywords))
            and message.reply_markup
        ):
            keys = [k for r in message.reply_markup.inline_keyboard for k in r]
            for k in keys:
                if "签到" in k.text and k.web_app:
                    url = k.web_app.url
                    bot_peer = await self.client.resolve_peer(self.bot_username)
                    url_auth = (
                        await self.client.invoke(
                            RequestWebView(peer=bot_peer, bot=bot_peer, platform="ios", url=url)
                        )
                    ).url
                    scheme = urlparse(url_auth)
                    params = parse_qs(scheme.fragment)
                    webapp_data = params.get("tgWebAppData", [""])[0]
                    token = await Link(self.client).captcha("ruji")
                    if not token:
                        self.log.warning("签到失败: 验证码解析失败, 正在重试.")
                        return await self.retry()
                    scheme = urlparse(url)
                    url_submit = scheme._replace(path="/api/checkin/verify", query="", fragment="").geturl()
                    origin = scheme._replace(path="/", query="", fragment="").geturl()
                    useragent = Faker().safari()

                    headers = {
                        "Content-Type": "application/json",
                        "Referer": url,
                        "Origin": origin,
                        "User-Agent": useragent,
                        "X-Requested-With": "XMLHttpRequest",
                    }

                    timestamp = int(datetime.now().timestamp())
                    nonce = self.generate_nonce()
                    signature = self.generate_signature(self.client.me.id, timestamp, nonce)

                    data = {
                        "user_id": str(self.client.me.id),
                        "token": token,
                        "signature": signature,
                        "timestamp": timestamp,
                        "webapp_data": webapp_data,
                        "nonce": nonce,
                    }
                    for i in range(10):
                        try:
                            async with httpx.AsyncClient(
                                http2=True, proxy=get_proxy_str(config.proxy)
                            ) as client:
                                resp = await client.post(url_submit, headers=headers, json=data)
                                result = resp.text

                                try:
                                    json_result = resp.json()
                                    if resp.status_code == 200:
                                        message = json_result.get("message", "签到成功")
                                        reward = json_result.get("reward", "")
                                        self.log.info(f"{message} {reward}")
                                        return
                                    elif resp.status_code == 409:
                                        detail = json_result.get("detail", "今日已经签到过了")
                                        self.log.info(detail)
                                        return await self.finish(RunStatus.NONEED, "今日已签到")
                                    else:
                                        detail = json_result.get("detail", "未知错误")
                                        self.log.info(detail)
                                        return await self.finish(RunStatus.ERROR, "签到失败")
                                except:
                                    self.log.warning(
                                        f"签到失败: 验证码识别后接口返回异常信息:\n{truncate_str(result, 100)}, 可能是您的请求 IP 风控等级较高导致的."
                                    )
                                    return await self.fail()

                        except (httpx.ProxyError, httpx.TimeoutException, OSError):
                            self.log.warning(
                                f"无法连接到站点的页面, 可能是您的网络或代理不稳定, 正在重试 ({i+1}/10)."
                            )
                            continue
                    else:
                        self.log.warning(f'无法连接到站点的页面: "{url_submit}", 可能是您的网络或代理不稳定.')
                        return await self.retry()
            else:
                self.log.warning(f"签到失败: 账户错误.")
                return await self.fail()

        if message.text and "请先点击下面加入我们的" in message.text:
            self.log.warning(f"签到失败: 账户错误.")
            return await self.fail()

        await super().message_handler(client, message)



from typing import List, Optional, Union
from loguru import logger
from pydantic import BaseModel, ValidationError

from ._smart import SmartMessager

__ignore__ = True


class TemplateBMessagerConfig(BaseModel):
    name: str = None
    chat_name: Union[str, int] = None  # 发送群聊名称
    style_messages: List[str] = None  # 使用的风格语料, 与 style_message_list 二选一
    style_message_list: str = (
        None  # 使用的风格语料列表, 例如 "some-wl@v1.yaml", 放置在 basedir 中, 且 @v1.yaml 尾缀是必须的
    )
    min_interval: int = None  # 发送最小间隔 (秒)
    max_interval: int = None  # 发送最大间隔 (秒)
    at: Optional[List[str]] = None  # 时间区间, 例如 ["5:00AM", "9:00PM"]
    msg_per_day: Optional[int] = None  # 每日发送次数
    min_msg_gap: int = 5  # 最小消息间隔
    force_day: bool = False  # 强制每条时间线在每个自然日运行
    prompt: Optional[str] = None  # 使用的提示词
    extra_prompt: Optional[str] = None  # 追加的提示词
    max_length: Optional[int] = 50  # 最大消息长度
    filter_recent_similarity: float = 0.6  # 过滤相似消息的相似度阈值
    max_count_recent_5: int = 1
    max_count_recent_10: int = 1

    # Backward compatibility
    interval: Optional[int] = None


class TemplateBMessager(SmartMessager):
    additional_auth = ["prime"]

    async def init(self):
        try:
            self.t_config = TemplateBMessagerConfig.model_validate(self.config)
        except ValidationError as e:
            self.log.warning(f"初始化失败: 水群器自定义模板 B 的配置错误:\n{e}")
            return False
        self.name = self.t_config.name or "自定义"
        self.chat_name = self.t_config.chat_name
        self.default_messages = self.t_config.style_message_list
        self.at = self.t_config.at
        self.msg_per_day = self.t_config.msg_per_day or self.msg_per_day
        self.min_msg_gap = self.t_config.min_msg_gap
        self.force_day = self.t_config.force_day
        self.max_length = self.t_config.max_length
        self.filter_recent_similarity = self.t_config.filter_recent_similarity
        self.max_count_recent_5 = self.t_config.max_count_recent_5
        self.max_count_recent_10 = self.t_config.max_count_recent_10
        # style_messages/ min_interval / max_interval / msg_per_day / prompt / extra_prompt 由 config 读取
        if not self.chat_name:
            self.log.warning(f"初始化失败: 没有定义任何目标群组, 请参考教程进行配置.")
            return False
        self.log = logger.bind(scheme="telemessager", name=self.name, username=self.me.full_name)
        return True


def use(**kw):
    return type("TemplatedClass", (TemplateBMessager,), kw)



import base64
import json
import os
from typing import List

from loguru import logger
from rich.prompt import Confirm, Prompt, IntPrompt, Confirm

from .utils import show_exception
from .config import ConfigManager, config
from .schema import Config, TelegramAccount, EmbyAccount
from .var import console
from .log import pad
from . import var, __url__


async def convert_session(accounts: List[TelegramAccount]):
    from .telegram.session import ClientsSession

    for a in accounts:
        success = False
        async with ClientsSession([a]) as clients:
            async for a, tg in clients:
                a.session = await tg.export_session_string()
                logger.info(f'账号 "{a.phone}" 登陆成功.')
                success = True
            if not success:
                logger.error(f'账号 "{a.phone}" 登陆失败, 已被禁用.')
                a.enabled = False
    return accounts


async def interactive_config(mongodb_url: str = None):
    from tomlkit import item

    cfg = Config()
    config.set(cfg)
    pad = " " * 23
    logger.info("我们将为您生成配置, 需要您根据提示填入信息, 并按回车确认.")
    logger.info(f"配置帮助详见: {__url__}.")
    logger.info(f"若需要重新开始, 请点击右上方的刷新按钮.")
    logger.info(f"若您需要更加高级的配置, 请使用右上角的 Config 按钮以修改配置文件.")

    if not mongodb_url:
        logger.info("请输入 MongoDB 连接地址 [dark_green](mongodb://user:pass@host:port)[/],")
        mongodb_url = Prompt.ask(
            pad + "回车以跳过, 但是可能导致记录的保活模拟设备环境变化:\n" + pad + "MongoDB URL",
            console=console,
        )
    if mongodb_url:
        cfg.mongodb = mongodb_url

    telegram_accounts = cfg.telegram.account
    while True:
        if len(telegram_accounts) > 0:
            logger.info(
                f"您当前填写了 {len(telegram_accounts)} 个 Telegram 账号信息: "
                f'{", ".join([t.phone for t in telegram_accounts])}'
            )
            more = Confirm.ask(pad + "是否继续添加?", default=False, console=console)
        else:
            more = Confirm.ask(pad + "是否添加 Telegram 账号?", default=True, console=console)
        if not more:
            break
        phone = Prompt.ask(
            pad + "请输入您的 Telegram 账号 (带国家区号) [dark_green](+861xxxxxxxxxx)[/]",
            console=console,
        )
        monitor = Confirm.ask(
            pad + "是否开启该账号的自动监控功能? (需要高级账号)", default=False, console=console
        )
        messager = Confirm.ask(
            pad + "是否开启该账号的自动水群功能? (需要高级账号)", default=False, console=console
        )
        telegram_accounts.append(TelegramAccount(phone=phone, monitor=monitor, messager=messager))
    if telegram_accounts:
        logger.info(f"即将尝试登录各账户并存储凭据, 请耐心等待.")
        await convert_session(telegram_accounts)

    emby_accounts = cfg.emby.account
    while True:
        if len(emby_accounts) > 0:
            logger.info(f"您当前填写了 {len(emby_accounts)} 个 Emby 账号信息.")
            more = Confirm.ask(pad + "是否继续添加?", default=False, console=console)
        else:
            more = Confirm.ask(pad + "是否添加 Emby 账号?", default=True, console=console)
        if not more:
            break
        url = Prompt.ask(
            pad + "请输入您的 Emby 站点 URL [dark_green](https://abc.com:443)[/]", console=console
        )
        username = Prompt.ask(pad + "请输入您在该 Emby 站点的用户名", console=console)
        password = Prompt.ask(
            pad + "请输入您在该 Emby 站点的密码 (不显示, 按回车确认)",
            password=True,
            console=console,
        )
        while True:
            time = Prompt.ask(
                pad + "设置模拟观看时长范围 (秒), 用空格分隔",
                default="120 240",
                show_default=True,
                console=console,
            )
            if " " in time:
                try:
                    time = [int(t) for t in time.split(None, 1)]
                    break
                except ValueError:
                    logger.warning(f"时长设置不正确, 请重新输入.")
            else:
                try:
                    time = int(time)
                    break
                except ValueError:
                    logger.warning(f"时长设置不正确, 请重新输入.")
        emby_accounts.append(EmbyAccount(url=url, username=username, password=password, time=time))
    advanced = Confirm.ask(pad + "是否配置高级设置", default=False, console=console)
    if advanced:
        while True:
            logger.info("发送关键日志消息到以下哪个账户?")
            logger.info(f"\t0. 不使用消息推送功能")
            for i, t in enumerate(telegram_accounts):
                logger.info(f"\t{i+1}. {t.phone}")
            selected = IntPrompt.ask(pad + "请选择", default=1, console=console)
            if selected:
                if selected > 0:
                    if selected <= len(telegram_accounts):
                        cfg.notifier.enabled = True
                        cfg.notifier.account = selected
                        break
                    else:
                        logger.warning("选择的账号不存在, 请重新选择.")
                else:
                    cfg.notifier.enabled = False
                    break
            else:
                logger.warning("选择的账号不存在, 请重新选择.")
        cfg.checkiner.timeout = IntPrompt.ask(
            pad + "设置每个 Telegram Bot 签到的最大尝试时间 (秒)",
            default=cfg.checkiner.timeout,
            show_default=True,
            console=console,
        )
        cfg.checkiner.retries = IntPrompt.ask(
            pad + "设置每个 Telegram Bot 签到的最大尝试次数",
            default=cfg.checkiner.retries,
            show_default=True,
            console=console,
        )
        cfg.checkiner.concurrency = IntPrompt.ask(
            pad + "设置最大可同时进行的 Telegram Bot 签到",
            default=cfg.checkiner.concurrency,
            show_default=True,
            console=console,
        )
        cfg.checkiner.random_start = IntPrompt.ask(
            pad + "设置计划任务时, 各站点之间签到的随机时间差异 (分钟)",
            default=cfg.checkiner.random_start,
            show_default=True,
            console=console,
        )
    content = item(json.loads(cfg.model_dump_json(exclude_none=True))).as_string().encode()
    content = base64.b64encode(content).decode()
    if var.use_mongodb_config:
        from .cache import cache

        cache.set("config", content)
        logger.info(f"您的配置已生成完毕并已存储到 MongoDB 数据库.")
    else:
        logger.info(
            f"您的配置[green]已生成完毕[/]! 您需要将以下内容写入托管平台的 EK_CONFIG 环境变量 ([red]SECRET[/]), 否则配置将在重启后丢失."
        )
        console.print()
        console.rule("EK_CONFIG")
        console.print(content)
        console.rule()
        console.print()
    start_now = Confirm.ask(pad + "是否立即启动?", default=True, console=console)
    if start_now:
        config.set(cfg)
        return True
    else:
        return False


async def prepare_config_str(config_str: str, mongodb_url: str = None):
    from tomlkit import item

    cfg = ConfigManager.load_config_str(config_str)
    cfg = ConfigManager.validate_config(cfg)
    config.set(cfg)
    if not cfg:
        return False

    # Add MongoDB check
    if (not cfg.mongodb) and (not mongodb_url):
        logger.warning("未设置 MongoDB 连接, 所有缓存数据将在重启后遗失或重置.")

    if mongodb_url:
        cfg.mongodb = mongodb_url

    to_login = []
    for a in cfg.telegram.account:
        if not a.session:
            to_login.append(a)
    if to_login:
        logger.info("即将尝试登陆各个账号.")
        await convert_session(to_login)
        content = item(json.loads(cfg.model_dump_json(exclude_none=True))).as_string().encode()
        content = base64.b64encode(content).decode()
        if var.use_mongodb_config:
            from .cache import cache

            cache.set("config", content)
            logger.info(f"您已登陆到 Telegram! 相关凭据已存储到 MongoDB 数据库.")
        else:
            logger.info(
                f"您已登陆到 Telegram! 您需要将以下内容重新写入托管平台的 EK_CONFIG 环境变量 ([red]SECRET[/]), "
                "否则登陆状态将在重启后丢失."
            )
            console.print()
            console.rule("EK_CONFIG")
            console.print(content)
            console.rule()
            console.print()
        start_now = Confirm.ask(pad + "是否立即启动?", default=True, console=console)
        if start_now:
            config.set(cfg)
            return True
        else:
            return False
    else:
        return True


async def prepare_mongodb_config(mongodb_url: str):
    logger.info(f"正在连接到 MongoDB 缓存, 请稍候.")
    config.set(Config())
    config.mongodb = mongodb_url
    var.use_mongodb_config = True
    try:
        from .cache import cache

        config_str = cache.get("config", None)
    except Exception as e:
        logger.error(f"您已在环境变量定义 MongoDB 地址, 但 MongoDB 缓存连接失败: {e}, 程序将退出.")
        show_exception(e, regular=False)
        return False

    if not config_str:
        return await interactive_config(mongodb_url)
    else:
        ok = await prepare_config_str(config_str, mongodb_url)
        return ok


async def public_preparation():
    env_config = os.environ.get(f"EK_CONFIG", None)
    mongodb_url = os.environ.get(f"EK_MONGODB", None)

    if mongodb_url:
        return await prepare_mongodb_config(mongodb_url)
    if env_config:
        return await prepare_config_str(env_config)
    else:
        return await interactive_config()



from urllib.parse import urlparse

from pyrogram.types import Message
from pyrogram.raw.functions.messages import RequestWebView
from faker import Faker
import httpx

from embykeeper.config import config
from embykeeper.utils import to_iterable, truncate_str, get_proxy_str
from embykeeper.runinfo import RunStatus

from ..link import Link
from ._templ_a import TemplateACheckin


class XiguaCheckin(TemplateACheckin):
    name = "西瓜"
    bot_username = "XiguaEmbyBot"
    bot_use_captcha = False
    bot_checkin_cmd = "/start"
    additional_auth = ["captcha"]
    templ_panel_keywords = ["冰镇西瓜"]

    async def message_handler(self, client, message: Message):
        text = message.caption or message.text
        if (
            text
            and any(keyword in text for keyword in to_iterable(self.templ_panel_keywords))
            and message.reply_markup
        ):
            keys = [k for r in message.reply_markup.inline_keyboard for k in r]
            for k in keys:
                if "签到" in k.text and k.web_app:
                    url = k.web_app.url
                    bot_peer = await self.client.resolve_peer(self.bot_username)
                    url_auth = (
                        await self.client.invoke(
                            RequestWebView(peer=bot_peer, bot=bot_peer, platform="ios", url=url)
                        )
                    ).url
                    token = await Link(self.client).captcha("xigua")
                    if not token:
                        self.log.warning("签到失败: 验证码解析失败, 正在重试.")
                        return await self.retry()
                    scheme = urlparse(url)
                    url_submit = scheme._replace(path="/api/checkin/verify", query="", fragment="").geturl()
                    origin = scheme._replace(path="/", query="", fragment="").geturl()
                    useragent = Faker().safari()
                    headers = {
                        "Content-Type": "application/json",
                        "Referer": url,
                        "Origin": origin,
                        "User-Agent": useragent,
                    }
                    data = {
                        "user_id": str(self.client.me.id),
                        "token": token,
                    }
                    for i in range(10):
                        try:
                            async with httpx.AsyncClient(
                                http2=True, proxy=get_proxy_str(config.proxy)
                            ) as client:
                                resp = await client.post(url_submit, headers=headers, json=data)
                                result = resp.text
                                if resp.status_code == 200:
                                    return
                                elif resp.status_code == 409:
                                    self.log.info(f"今日已经签到过了.")
                                    return await self.finish(RunStatus.NONEED, "今日已签到")
                                else:
                                    self.log.warning(
                                        f"签到失败: 验证码识别后接口返回异常信息:\n{truncate_str(result, 100)}, 可能是您的请求 IP 风控等级较高导致的."
                                    )
                                    return await self.fail()
                        except (httpx.ProxyError, httpx.TimeoutException, OSError):
                            self.log.warning(
                                f"无法连接到站点的页面, 可能是您的网络或代理不稳定, 正在重试 ({i+1}/10)."
                            )
                            continue
                    else:
                        self.log.warning(f'无法连接到站点的页面: "{url_submit}", 可能是您的网络或代理不稳定.')
                        return await self.retry()
            else:
                self.log.warning(f"签到失败: 账户错误.")
                return await self.fail()

        if message.text and "请先点击下面加入我们的" in message.text:
            self.log.warning(f"签到失败: 账户错误.")
            return await self.fail()

        await super().message_handler(client, message)



import asyncio

from embykeeper.utils import to_iterable

from ..link import Link
from . import BotCheckin

__ignore__ = True


class EPubGroupChatCheckin(BotCheckin):
    name = "EPub 电子书库群组每日发言"
    chat_name = "libhsulife"
    additional_auth = ["prime"]
    bot_use_captcha = False

    async def send_checkin(self, retry=False):
        for _ in range(3):
            times = self.config.get("times", 5)
            min_letters = self.config.get("letters", 7)
            prompt = self.config.get(
                "prompt",
                f"请输出{times}行的诗，所有行都至少{min_letters}个字，必须严格遵守每行字数要求！最多{times+2}行，只输出古诗内容，禁止输出其他提示语言，禁止输出逗号句号，每行开头必须有标号'@@@'",
            )
            answer, by = await Link(self.client).gpt(prompt)
            if not answer:
                continue
            lines = [
                l.lstrip("@@@").strip()
                for l in answer.splitlines()
                if l.startswith("@@@") and len(l.strip()) >= min_letters + 3
            ]
            if len(lines) > times + 2 or len(lines) < times:
                continue
            else:
                for l in lines:
                    self.log.info(f"即将向群组发送水群消息: {l}.")
                await asyncio.sleep(10)
                cmds = to_iterable(lines)
                for i, cmd in enumerate(cmds):
                    if retry and not i:
                        await asyncio.sleep(self.bot_retry_wait)
                    if i < len(cmds):
                        await asyncio.sleep(self.bot_send_interval)
                    await self.send(cmd)
                await self.finish(message="已发送发言")
                return
        else:
            return await self.fail(message="无法生成发言内容")

    async def message_handler(*args, **kw):
        return
